<?php
// +----------------------------------------------------------------------+
// | HIMMEL Core                                     2004-07-23T21:31:00Z |
// |                                                        For PHP 5.0.0 |
// +----------------------------------------------------------------------+
// | Distributed at http://www.arielworks.net/works/codeyard/himmel       |
// +----------------------------------------------------------------------+



class applyS3
{
    protected $id;
    protected $sdf_source;
    protected $processor_root;
    protected $xhtml_version;
    protected $result_source;

    function applyS3($id, $sdf_source, $processor_root)
    {
        require_once(dirname(__FILE__) . "/lemmih.inc");

        $this->id             = $id;
        $this->sdf_source     = $sdf_source;
        $this->processor_root = $processor_root;

        /* Select XHTML version */
        $request_headers = apache_request_headers();
        $accept = explode(",", $request_headers["Accept"]);
        if (in_array("application/xhtml+xml", $accept)) {
            $this->xhtml_version = (double) 1.1;
        } else {
            $this->xhtml_version = (double) 1.0;
        }

        /* Translate into XHTML */
        $this->_translation();

    }

    function _translation()
    {
        /* Apply S3 */
        $param_S3 = array(array("http://www.arielworks.net/works/codeyard/sdf/s3", "processor_root", $this->processor_root),
                          array("http://www.arielworks.net/works/codeyard/sdf/s3", "id", $this->id));
        $result = $this->_smartXSLT($this->sdf_source, dirname(__FILE__)  . "/s3/interface.xsl", $param_S3);

        /* For old brosers */
        if ($this->xhtml_version == 1.0) {
            $result = $this->_smartXSLT($result, dirname(__FILE__) . "/11to10.xsl");
        }

        /* Strip space nodes and indent */
        $indent = new DomDocument();
        $indent->preserveWhiteSpace = FALSE;
        $indent->loadXML($result);
        $result = $indent->saveXML();
        $result = $this->_smartXSLT($result, dirname(__FILE__) . "/indent.xsl");

        /* Clean up namespace and set DOCTYPE */
        if ($this->xhtml_version == 1.0) {
            $result = $this->_smartXSLT($result, dirname(__FILE__) . "/valid10.xsl");
        } else {
            $result = $this->_smartXSLT($result, dirname(__FILE__) . "/valid11.xsl");
        }

        /* Hack for IE6 */
        if (preg_match("/MSIE 6.0/", $_SERVER["HTTP_USER_AGENT"])) {
            $result = preg_replace("/<\?xml.*?\?>(\n)?/", "", $result);
        }

        /* Hacks */
        $result = preg_replace("/<textarea([^>]*?)\/>/is", '<textarea$1></textarea>', $result);
        $result = preg_replace("/<script([^>]*?)\/>/is", '<script$1></script>', $result);
        $result = preg_replace("/(<[^>]*?[^ ])\/>/is", "$1 />", $result);

        $this->result_source = $result;
    }

    function _smartXSLT($xmls, $xsls, $param = array())
    {
        /* Load the sources */
        if (is_file($xmls)) {
            $xml = new DomDocument;
            $xml->load($xmls);
        } elseif(is_string($xmls)) {
            $xml = new DomDocument;
            $xml->loadXML($xmls);
        } elseif(is_object($xmls)) {
            $xml = $xmls;
        }
        if (is_file($xsls)) {
            $xsl = new DomDocument;
            $xsl->load($xsls);
        } elseif(is_string($xsls)) {
            $xsl = new DomDocument;
            $xsl->loadXML($xsls);
        } elseif(is_object($xsls)) {
            $xsl = $xsls;
        }

        /* XSLT */
        $proc = new xsltProcessor;
        $proc->importStyleSheet($xsl);
        foreach ($param as $param_item) {
            $proc->setParameter($param_item[0], $param_item[1], $param_item[2]);
        }
        return $proc->transformToXML($xml);
    }

    function saveResult()
    {
        return $this->result_source;
    }
}

class HIMMEL extends applyS3
{
    protected $modified;
    var $if_modified_since = TRUE;

    function HIMMEL($id, $sdf_source, $processor_root)
    {
        $this->applyS3($id, $sdf_source, $processor_root);
        $this->_getModified();
    }

    function _getModified()
    {
        $dom_mod = new DomDocument;
        $dom_mod->loadXML($this->result_source);
        $xph_mod = new domXPath($dom_mod);
        $xph_mod->registerNamespace("xh", "http://www.w3.org/1999/xhtml");
        $node_mod = $xph_mod->query("/xh:html/xh:head/xh:meta[./@name = 'WWWC']");
        $modified = $node_mod->item(0)->getAttribute("content");
        $modified = str_replace("/", "-", substr($modified, 0, 10)) . "T" . substr($modified, -5, 5) . "Z";
        $modified = parse_w3cdtf($modified);
        $this->modified = $modified["timestamp"];
    }

    function process()
    {
        $this->sendHTTPHeaders();

        if ($this->if_modified_since) {
            /* If-Modified-Since */
            $request_headers = apache_request_headers();
            $since = parse_http_date($request_headers["If-Modified-Since"]);
            if ($request_headers["If-Modified-Since"] && $since["timestamp"] >= $this->modified) {
                header("HTTP/1.1 304 Not Modified");
                return;
            }
        }
        print $this->saveResult();
    }

    function sendHTTPHeaders()
    {
        if ($this->xhtml_version >= 1.1) {
            header("Content-Type: application/xhtml+xml; charset=UTF-8");
        } elseif ($this->xhtml_version == 1.0) {
            header("Content-Type: text/html; charset=UTF-8");
        }
        header("Last-Modified: " . gmdate("D, d M Y H:i:s", $this->modified) . " GMT");
        header("Content-Style-Type: text/css");
        header("Content-Script-Type: text/javascript");
    }
}
?>